@@ -1,4 +1,3 @@
-# -*- coding: UTF-8 -*-
 """Defaults for Germinate."""
 
 # Copyright (c) 2011 Canonical Ltd.
@@ -19,13 +18,13 @@
 # 02110-1301, USA.
 
 # Where do we get up-to-date seeds from?
-seeds = 'http://people.canonical.com/~ubuntu-archive/seeds/'
-seeds_bzr = 'http://bazaar.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/'
-seeds_git = 'git://git.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/+git/'
-release = 'ubuntu.wily'
+seeds = "https://ubuntu-archive-team.ubuntu.com/seeds/"
+seeds_bzr = "http://bazaar.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/"
+seeds_git = "git://git.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/+git/"
+release = "ubuntu.mantic"
 
 # If we need to download Packages.gz and/or Sources.gz, where do we get
 # them from?
-mirror = 'http://archive.ubuntu.com/ubuntu/'
-dist = 'wily'
-arch = 'i386'
+mirror = "http://archive.ubuntu.com/ubuntu/"
+dist = "mantic"
+arch = "amd64"
