@@ -180,9 +180,23 @@ def main(argv):
                         break
             if seed_name not in mapped_seeds:
                 mapped_seeds.append(seed_name)
-        packages = []
-        for mapped_seed in mapped_seeds:
-            packages.extend(germinator_method(structure, mapped_seed))
+
+        iter_mapped_seeds = iter(mapped_seeds)
+        packages = germinator_method(structure, next(iter_mapped_seeds))
+        for mapped_seed in iter_mapped_seeds:
+            value = germinator_method(structure, mapped_seed)
+            if isinstance(value, dict):
+                intersection = set(packages).intersection(value.keys())
+                if intersection:
+                    error_exit(f"Not supported: Merging keys {intersection!r}")
+                packages.update(value)
+            elif isinstance(value, list):
+                packages.extend(value)
+            else:
+                raise NotImplementedError(
+                    "seed_packages: support for type '{type(value)}' not implemented"
+                )
+
         return packages
 
     def metapackage_name(structure, seed_name):
@@ -338,8 +352,8 @@ def main(argv):
             packages = seed_packages(
                 germinator.get_seed_entries, structure, seed_name
             )
-            alternatives = germinator.get_seed_alternatives(
-                structure, seed_name
+            alternatives = seed_packages(
+                germinator.get_seed_alternatives, structure, seed_name
             )
             for package in packages:
                 if package == meta_name:
@@ -392,7 +406,13 @@ def main(argv):
                         "debootstrap)" % (seed_name, architecture, package)
                     )
                 else:
-                    new_recommends_list.add(package)
+                    try:
+                        new_recommends_list.add(
+                            "%s | %s"
+                            % (package, " | ".join(alternatives[package]))
+                        )
+                    except KeyError:
+                        new_recommends_list.add(package)
 
             seed_name_recommends = "%s-recommends" % seed_name
             output_recommends_filename = os.path.join(
